package me.teridax.jcash;

import me.teridax.jcash.gui.Loader;
import me.teridax.jcash.gui.account.AccountController;
import me.teridax.jcash.gui.login.LoginController;

import javax.swing.*;

public final class Main {

    /**
     * Main instance of this program. Contains the primary window.
     */
    private static Main instance;

    /**
     * Primary window of this program
     */
    private final JFrame window;

    private Main() {
        // create main window and set defaults
        this.window = new JFrame();
        this.window.setTitle("Bankautomat");
        this.window.setLocationByPlatform(true);
        this.window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    public static void main(String[] args) {
        // create main instance and show the login screen
        instance();
        getInstance().showLoginScreen();
    }

    public static Main getInstance() {
        return instance;
    }

    /**
     * Attempts to create a new instance of the singleton class Main.
     * This method throws an exception if the class is already instantiated.
     * @throws IllegalStateException if the class is already instantiated
     * @pre the static instance of this class should be null.
     * @post the static instance of this class will be set
     */
    public static void instance() {
        if (null != instance)
            throw new IllegalStateException(Main.class.getName() + " is already initialized");

        Main.instance = new Main();
    }

    /**
     * Shows the open dialog for selecting a database file. After selection, it then proceeds to prompt login.
     * Afterward the selected account can be managed.
     * This method is non-blocking and all work described is performed asynchronously on the AWT Event dispatcher.
     */
    public void showLoginScreen() {
        SwingUtilities.invokeLater(() -> {
            try {
                // select db file
                var path = Loader.load();
                // read database and login
                var login = new LoginController(path);

                // when we have logged in set the account viewer as window content
                login.addAccountSelectionListener(account -> {
                    var profileCont = new AccountController(account);
                    this.window.setContentPane(profileCont.getView());
                    this.window.revalidate();
                    this.window.repaint();
                });

                // we are not logged in yet, so show the login prompt on the main window
                this.window.setContentPane(login.getView());
                this.window.setSize(800, 600);
                this.window.setVisible(true);

            } catch (IllegalStateException e) {
                System.out.println("no file selected. goodbye");
            }
        });
    }

    /**
     * Logs the user out of the database, hiding the main window.
     */
    public void logout() {
        window.setContentPane(new JLabel("you're logged out"));
        window.setVisible(false);
    }
}